<?php

namespace Vaptreporting\Importreport\Http\Controllers;

use App\Http\Controllers\ResourceController as BaseController;
use Form;
use Vaptreporting\Importreport\Http\Requests\InfoRequest;
use Vaptreporting\Importreport\Interfaces\InfoRepositoryInterface;
use Vaptreporting\Importreport\Models\Info;

/**
 * Resource controller class for info.
 */
class InfoResourceController extends BaseController
{

    /**
     * Initialize info resource controller.
     *
     * @param type InfoRepositoryInterface $info
     *
     * @return null
     */
    public function __construct(InfoRepositoryInterface $info)
    {
        parent::__construct();
        $this->repository = $info;
        $this->repository
            ->pushCriteria(\Litepie\Repository\Criteria\RequestCriteria::class)
            ->pushCriteria(\Vaptreporting\Importreport\Repositories\Criteria\InfoResourceCriteria::class);
    }

    /**
     * Display a list of info.
     *
     * @return Response
     */
    public function index(InfoRequest $request)
    {
        $view = $this->response->theme->listView();

        if ($this->response->typeIs('json')) {
            $function = camel_case('get-' . $view);
            return $this->repository
                ->setPresenter(\Vaptreporting\Importreport\Repositories\Presenter\InfoPresenter::class)
                ->$function();
        }

        $infos = $this->repository->paginate();

        return $this->response->title(trans('importreport::info.names'))
            ->view('importreport::info.index', true)
            ->data(compact('infos'))
            ->output();
    }

    /**
     * Display info.
     *
     * @param Request $request
     * @param Model   $info
     *
     * @return Response
     */
    public function show(InfoRequest $request, Info $info)
    {

        if ($info->exists) {
            $view = 'importreport::info.show';
        } else {
            $view = 'importreport::info.new';
        }

        return $this->response->title(trans('app.view') . ' ' . trans('importreport::info.name'))
            ->data(compact('info'))
            ->view($view, true)
            ->output();
    }

    /**
     * Show the form for creating a new info.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(InfoRequest $request)
    {

        $info = $this->repository->newInstance([]);
        return $this->response->title(trans('app.new') . ' ' . trans('importreport::info.name')) 
            ->view('importreport::info.create', true) 
            ->data(compact('info'))
            ->output();
    }

    /**
     * Create new info.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(InfoRequest $request)
    {
        try {
            $attributes              = $request->all();
            $attributes['user_id']   = user_id();
            $attributes['user_type'] = user_type();
            $info                 = $this->repository->create($attributes);

            return $this->response->message(trans('messages.success.created', ['Module' => trans('importreport::info.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('importreport/info/' . $info->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/importreport/info'))
                ->redirect();
        }

    }

    /**
     * Show info for editing.
     *
     * @param Request $request
     * @param Model   $info
     *
     * @return Response
     */
    public function edit(InfoRequest $request, Info $info)
    {
        return $this->response->title(trans('app.edit') . ' ' . trans('importreport::info.name'))
            ->view('importreport::info.edit', true)
            ->data(compact('info'))
            ->output();
    }

    /**
     * Update the info.
     *
     * @param Request $request
     * @param Model   $info
     *
     * @return Response
     */
    public function update(InfoRequest $request, Info $info)
    {
        try {
            $attributes = $request->all();

            $info->update($attributes);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('importreport::info.name')]))
                ->code(204)
                ->status('success')
                ->url(guard_url('importreport/info/' . $info->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('importreport/info/' . $info->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the info.
     *
     * @param Model   $info
     *
     * @return Response
     */
    public function destroy(InfoRequest $request, Info $info)
    {
        try {

            $info->delete();
            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('importreport::info.name')]))
                ->code(202)
                ->status('success')
                ->url(guard_url('importreport/info/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('importreport/info/' . $info->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove multiple info.
     *
     * @param Model   $info
     *
     * @return Response
     */
    public function delete(InfoRequest $request, $type)
    {
        try {
            $ids = hashids_decode($request->input('ids'));

            if ($type == 'purge') {
                $this->repository->purge($ids);
            } else {
                $this->repository->delete($ids);
            }

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('importreport::info.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('importreport/info'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/importreport/info'))
                ->redirect();
        }

    }

    /**
     * Restore deleted infos.
     *
     * @param Model   $info
     *
     * @return Response
     */
    public function restore(InfoRequest $request)
    {
        try {
            $ids = hashids_decode($request->input('ids'));
            $this->repository->restore($ids);

            return $this->response->message(trans('messages.success.restore', ['Module' => trans('importreport::info.name')]))
                ->status("success")
                ->code(202)
                ->url(guard_url('/importreport/info'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->status("error")
                ->code(400)
                ->url(guard_url('/importreport/info/'))
                ->redirect();
        }

    }

}
